@@ -1,7 +1,7 @@
 package Perinci::Sub::Complete;
 
-our $DATE = '2015-03-09'; # DATE
-our $VERSION = '0.75'; # VERSION
+our $DATE = '2015-03-30'; # DATE
+our $VERSION = '0.76'; # VERSION
 
 use 5.010001;
 use strict;
@@ -10,7 +10,7 @@ use experimental 'smartmatch';
 use Log::Any '$log';
 
 use Complete;
-use Complete::Util qw(hashify_answer complete_array_elem);
+use Complete::Util qw(hashify_answer complete_array_elem combine_answers);
 use Perinci::Sub::Util qw(gen_modified_sub);
 
 require Exporter;
@@ -117,13 +117,30 @@ sub complete_from_schema {
             $static++;
             return; # from eval. there should not be any other value
         }
-        if ($type =~ /\Abool\*?\z/) {
+        if ($type eq 'any') {
+            # because currently Data::Sah::Normalize doesn't recursively
+            # normalize schemas in 'of' clauses, etc.
+            require Data::Sah::Normalize;
+            if ($cs->{of} && @{ $cs->{of} }) {
+                $fres = combine_answers(
+                    grep { defined } map {
+                        complete_from_schema(
+                            schema=>Data::Sah::Normalize::normalize_schema($_),
+                            word => $word,
+                            ci => $ci,
+                        )
+                    } @{ $cs->{of} }
+                );
+                goto RETURN_RES; # directly return result
+            }
+        }
+        if ($type eq 'bool') {
             $log->tracef("[comp][periscomp] adding completion from possible values of bool");
             push @$words, 0, 1;
             $static++;
             return; # from eval
         }
-        if ($type =~ /\Aint\*?\z/) {
+        if ($type eq 'int') {
             my $limit = 100;
             if ($cs->{between} &&
                     $cs->{between}[0] - $cs->{between}[0] <= $limit) {
@@ -185,7 +202,7 @@ sub complete_from_schema {
             }
             return; # from eval
         }
-        if ($type =~ /\Afloat\*?\z/) {
+        if ($type eq 'float') {
             if (length($word) && $word !~ /\A-?\d*(\.\d*)?\z/) {
                 $log->tracef("[comp][periscomp] word not a float");
                 $words = [];
@@ -217,6 +234,8 @@ sub complete_from_schema {
         }
     }; # eval
 
+    $log->tracef("[periscomp] complete_from_schema died: %s", $@) if $@;
+
     goto RETURN_RES unless $words;
     $fres = hashify_answer(
         complete_array_elem(array=>$words, word=>$word, ci=>$ci),
@@ -902,7 +921,7 @@ Perinci::Sub::Complete - Complete command-line argument using Rinci metadata
 
 =head1 VERSION
 
-This document describes version 0.75 of Perinci::Sub::Complete (from Perl distribution Perinci-Sub-Complete), released on 2015-03-09.
+This document describes version 0.76 of Perinci::Sub::Complete (from Perl distribution Perinci-Sub-Complete), released on 2015-03-30.
 
 =head1 SYNOPSIS
 
@@ -1282,7 +1301,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/Perinci-Su
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/perlancar/perl-Perinci-Sub-Complete>.
+Source repository is at L<https://github.com/sharyanto/perl-Perinci-Sub-Complete>.
 
 =head1 BUGS
 
